#!/bin/bash

main()
{

if [ $# -gt 0 ]; then

  echo "This script does not accept arguments, parameters need to be added to config.bash Aborting"
  exit 1

fi

#Get script directory
dir=$(pwd)

echo
echo "Using the following configuration:"
echo
source "$dir"/config.bash
grep -o '^[^#]*' config.bash
echo

Init
Test_arguments
Test_Comp
Create_metrics

}


Job_completed()
{
  if [ "$Jobs_scheduler" == "slurm" ]; then
    local id1=${1##* }
    sleep 5
    if ! scontrol show job $id1 | grep -q 'JobState=COMPLETED'; then
      Completed=false
    else
      Completed=true
    fi	

  elif [ "$Jobs_scheduler" == "lsf" ]; then
    local id1=$(head -n1 1 | cut -d'<' -f2 | cut -d'>' -f1)
    sleep 5
    if ! bjobs -l $id | grep -q 'Status <DONE>'; then
      Completed=false
    else
      Completed=true
    fi
  elif [ "$Jobs_scheduler" == "torque" ]; then
    local id1=$(head -n1 $1 | awk '{ print $3 }')
    sleep 5
    if ! qstat f $id | grep -q 'exit_status = 0'; then
      Completed=false
    else
      Completed=true
    fi
  fi
  

}

Init()
{
  #Init variables with default values in case of missing
  Nemo_path="${Nemo_path:-"."}"
  Nemo_cores="${Nemo_cores:-( 48 )}"
  Jobs_n_cores="${Jobs_n_cores:-48}"
  Jobs_scheduler="${Jobs_scheduler:-"slurm"}"
  time="${Jobs_time:-"0"}"
  queue="${Jobs_queue:-""}"
  compile="${Compilation_compile:-"false"}"
  cfg="${Compilation_ref:-"ORCA2_ICE_PISCES"}"
  arch="${Compilation_arch:-"X64_MN4"}"
  name_cfg="${Compilation_name:-"ORCA2_EXTRAE"}"
  comp_cfg="${Compilation_sub:-""}"
  Modules="${Modules:-""}"
}


# Checks if the paths given are correct.

Test_arguments()
{
  # Nemo path correct?
  if ! test -d "${Nemo_path}"; then
    echo "Nemo relative path: ${Nemo_path} is not found"
    echo
    exit 1
  fi
  #Nemo_cores is array?
  if [[ ! "$(declare -p Nemo_cores)" =~ "declare -a" ]]; then
    echo "Error Nemo_cores has to be a bash array like ( 4 24 48 )"
    echo
    exit 1
  fi
  # cfg exists?
  if ! test -d "${Nemo_path}/cfgs/${cfg}"; then
    echo "configuration: ${cfg} doesn't exists in ${Nemo_path}/cfgs dir"
    echo
    exit 1
  fi
  # arch exists?
  if ! test -f "${Nemo_path}/arch/arch-${arch}.fcm"; then
    echo "architecture: arch-${arch}.fcm doesn't exists in ${Nemo_path}/arch dir"
    echo
    exit 1
  fi
  # scheduler correct?
  if [ "$Jobs_scheduler" != "slurm" ] && [ "$Jobs_scheduler" != "lsf" ] && [ "$Jobs_scheduler" != "torque" ]; then
    echo "$Jobs_scheduler is not a valid scheduler"
    echo
    exit 1
  fi
  #Nemo_cores is array?
  if [[ ! "$(declare -p Nemo_cores)" =~ "declare -a" ]]; then
    echo "Error, variable Nemo_cores has to be an array"
    echo
    exit 1
  fi
  #Modules available
  if ! module load $Modules;then
    echo "Error loading modules aborting"
    echo
    exit 1
  fi
  echo
  #$EXTRAE_HOME loaded ?
  if ! test -d "${EXTRAE_HOME}"; then
  echo "Extrae relative path: ${EXTRAE_HOME} is not found"
    echo
    exit 1
  else
    sed -i 's|home=.*|home="'"$EXTRAE_HOME"'"|g' extrae.xml
  fi

  # Adding -d to variable if not empty
  if [ -n "$comp_cfg" ]; then
    comp_cfg="-d $comp_cfg"
  fi
  
  # Creating auxiliar vars for submiting jobs
  if [ "$Jobs_scheduler" == "slurm" ]; then 
    job="sbatch"
  elif [ "$Jobs_scheduler" == "lsf" ]; then
    job="bsub"
  elif [ "$Jobs_scheduler" == "torque" ]; then
    job="qsub"
  fi

  
}



Test_Comp()
{

  # Checking if compilation is needed

  if [ "$compile" == true ]; then
      echo 'compile parameter is inicialized true'
  fi
  
  # Get text lines corresponding to the flags
  line=$(sed -n '/^%FCFLAGS /p' "$Nemo_path"/arch/arch-"${arch}".fcm)
  line2=$(sed -n '/^%FPPFLAGS /p' "$Nemo_path"/arch/arch-"${arch}".fcm)
  line3=$(sed -n '/^%LDFLAGS /p' "$Nemo_path"/arch/arch-"${arch}".fcm)
   
  # If -g is not there, recompilation is requiered and -g added
  if ! echo "${line}"|grep -q "\-g\b"; then
    echo "-g flag not found in arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -g /' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi

  # If finstrument-functions is not there recompilation is requiered and -finstrument-functions added
  if ! echo "${line}"|grep -q "\-finstrument-functions\b"; then
    echo "-finstrument-functions flag not found in arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -finstrument-functions/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi

  # If nemo executable is not on the run file compile
  
  if ! test -f "${Nemo_path}/cfgs/${name_cfg}/EXP00/nemo"; then
    echo "nemo executable not found in cfg"
    compile=true
  fi
  
  # If -pg is not there recompilation is requiered and -pg added

  if ! echo "${line}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in FCFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -pg/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi

  if ! echo "${line2}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in FPPFLAGS arch-${arch}.fcm : editing arch-${arch}.fcm "
    sed -i '/^%FPPFLAGS/ s/$/ -pg/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi

  if ! echo "${line3}"|grep -q "\-pg\b"; then
    echo "-pg flag not found in LDFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%LDFLAGS/ s/$/ -pg/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi


  # If -rdynamic is not there recompilation is requiered and -rdynamic added
  
  if ! echo "${line}"|grep -q "\-rdynamic\b"; then
    echo "-rdynamic flag not found in FCFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%FCFLAGS/ s/$/ -rdynamic/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi

 if ! echo "${line3}"|grep -q "\-export-dynamic\b"; then
    echo "-export-dynamic flag not found in LDFLAGS arch-${arch}.fcm: editing arch-${arch}.fcm "
    sed -i '/^%LDFLAGS/ s/$/ -export-dynamic/' "${Nemo_path}"/arch/arch-"${arch}".fcm
    compile=true
  fi
  
  

  #Compile the program if needed

  if [ $compile == true ]; then
    echo "Compiling Nemo, expected duration 35m"
    echo "Output of the compilation in compile.err and compile.out"

    printf -v workload1 "cd ${Nemo_path}\n./makenemo -r ${cfg} -n ${name_cfg} -m ${arch} -j$Jobs_n_cores $comp_cfg"
    python3 Job_Creator.py -f "compile" -j "compile" --set-core "${Jobs_n_cores}" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "${workload1}"
    
    state1=$("$job" --wait compile."$Jobs_scheduler")
    echo
    Job_completed "$state1"
    if [ $Completed == false ]; then
      echo "Nemo compilation failed, remember to load all the needed modules. Check the details in compile.err"
      echo
      exit 1
    else 
      echo "Nemo compilation successful"
      echo
    fi
    
  else
    echo "Compilation not needed"  
    echo
  fi

   
  #Copy all the EXP00 data but don't overwrite namelist just the executable
  cd "$dir" || echo "Error original dir doesn't exist" exit
  cp -n  "${Nemo_path}"/cfgs/"${name_cfg}"/EXP00/* .
  cp "${Nemo_path}"/cfgs/"${name_cfg}"/EXP00/nemo .

  if [[ $comp_cfg == "-d OCE del_key 'key_si3 key_top'" ]]; then
    sed -i '/_def_nemo-ice.xml\|def_nemo-pisces.xml/d' context_nemo.xml #DELETE ICE AND PISCES CONTEXT (NOT USED)
  fi

  #Solving NEMO input file common errors

  if test -f "weights_core_orca2_bicubic_noc.nc"; then
    mv weights_core_orca2_bicubic_noc.nc weights_core2_orca2_bicub.nc #RENAME WRONG NAMED FILES
  fi

  if test -f "weights_core_orca2_bilinear_noc.nc"; then 
    mv weights_core_orca2_bilinear_noc.nc weights_core2_orca2_bilin.nc #RENAME WRONG NAMED FILES
  fi

  
}

Create_metrics()
{


  #Changing iterations, big traces generate problems.
  sed -i 's|nn_itend * =.*|nn_itend    =       12  !  last  time step (std 5475)|g' namelist_cfg

  #Generating function list in case of missing

  if ! test -f "extrae_functions_for_xml.txt"; then
    rm gmon* 2> /dev/null
    echo "Runing Nemo with 2 cores to obtain function data..."
    echo
    python3 Job_Creator.py -f "run" -j "run" --set-core 4 -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "mpirun -np 2 ./nemo" 
    
    state2=$("$job" --wait run."$Jobs_scheduler")
    Job_completed "$state2"
    if [ $Completed == false ]; then
      echo "Nemo execution failed look at run.err and ocean.output for more info"
      echo "Remember that the namelist files are now the default, change theme in order to fit with the input files in the dir " 
      echo
      exit 1
    else 
      echo "Gprof files generated "
      echo
    fi
    echo "Gthrottling functions ..."
    echo
    python3 Job_Creator.py -f "gthrottling" -j "gthrottling" --set-core 4 -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "./gthrottling.sh nemo" 
    state3=$("$job" --wait gthrottling."$Jobs_scheduler")
    Job_completed "$state3"
    if [ $Completed == false ]; then
      echo "Error listing functions, look at gthrottling.err for more info"
      echo
      exit 1
    else
      echo "Functions listed correctly"
      echo
    fi

    ./extraf.sh  nemo extrae_functions.txt

  else
    echo "Functions already listed, file extrae_functions_for_xml.txt does exist"
    echo
  fi

  sed -i "s|list=.*|list=\"${dir}/extrae_functions_for_xml.txt\" exclude-automatic-functions=\"yes\">|g" extrae.xml
  
  # Run nemo with extrae

  for core in "${Nemo_cores[@]}" 
  
  do
    
    echo "Creating trace with $core cores..."
    echo
    python3 Job_Creator.py -f "run_extrae" -j "run_extrae" --set-core "$core" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w "mpirun -np $core ./trace.sh ./nemo" 
    
    state4=$("$job" --wait run_extrae."$Jobs_scheduler")
    Job_completed "$state4"
    if [ $Completed == false ]; then
      echo "Nemo execution failed no traces files generated more info inside run_extrae.err"
      echo 
      exit 1
    fi
    mv nemo.prv nemo_"$core".prv
    mv nemo.pcf nemo_"$core".pcf
    mv nemo.row nemo_"$core".row	
    echo "Cutting best iteration"
    echo
    magiccut/./magicCut nemo_"${core}".prv 12 > cut_"$core".out 2>&1
    if ! ls nemo_"$core".best_cut.prv; then
      echo "Cut failed, aborting." 
      echo
      exit 1
    fi
    echo
    # Creating folder
    if ! test -d "Metrics"; then
      mkdir Metrics
    fi

    cp nemo_"$core".best_cut.* Metrics
 done
  
  # Create performance metrics


  echo "Creating metrics and storing theme in Metrics folder"
  echo
  python3 Job_Creator.py -f "analysis" -j "analysis" --set-core "${Jobs_n_cores}" -s "$Jobs_scheduler" --set-time "$time" --set-queue "$queue" -w ".././modelfactors.py *" 
  mv analysis."$Jobs_scheduler" Metrics
  cd Metrics||(echo "Error Metrics folder doesn't exists";  exit 1)
  state5=$("$job" --wait analysis."$Jobs_scheduler")
  Job_completed "$state5"
  if [ $Completed == false ]; then
    echo "Error, metrics have not generated check Metrics/analysis.err to get more details"
    echo
    exit 1
 
  fi
  
  echo "------------------------------------------------------------------------------"
  echo "------------------------- Script Completed -----------------------------------"
  echo "----------------------- Data in Metrics folder -------------------------------"
  echo "------------------------------------------------------------------------------"
  echo "------------------------------------------------------------------------------"
  echo
}
main "$@"; exit
